@@ -432,8 +432,10 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
 {
   GstStructure *s;
   gchar lang_code[3] = { '\0', '\0', '\0' }, *t;
+  pgc_t *pgc0;
   gint title_set_nr;
   gint num_titles;
+  gint pgn0, pgc0_id;
   gint i;
 
   /* make sure our title number is valid */
@@ -489,9 +491,18 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
   s = gst_structure_new ("application/x-gst-dvd",
       "event", G_TYPE_STRING, "dvd-lang-codes", NULL);
 
+  /* so we can filter out invalid/unused streams (same for all chapters) */
+  cur_title_get_chapter_pgc (src, 0, &pgn0, &pgc0_id, &pgc0);
+
   /* audio */
   for (i = 0; i < src->vts_file->vtsi_mat->nr_of_vts_audio_streams; i++) {
-    const audio_attr_t *a = &src->vts_file->vtsi_mat->vts_audio_attr[i];
+    const audio_attr_t *a;
+
+    /* audio stream present? */
+    if (pgc0 != NULL && (pgc0->audio_control[i] & 0x8000) == 0)
+      continue;
+
+    a = &src->vts_file->vtsi_mat->vts_audio_attr[i];
 
     t = g_strdup_printf ("audio-%d-format", i);
     gst_structure_set (s, t, G_TYPE_INT, (int) a->audio_format, NULL);
@@ -513,7 +524,13 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
 
   /* subtitle */
   for (i = 0; i < src->vts_file->vtsi_mat->nr_of_vts_subp_streams; i++) {
-    const subp_attr_t *u = &src->vts_file->vtsi_mat->vts_subp_attr[i];
+    const subp_attr_t *u;
+
+    /* subpicture stream present? */
+    if (pgc0 != NULL && (pgc0->subp_control[i] & 0x80000000) == 0)
+      continue;
+
+    u = &src->vts_file->vtsi_mat->vts_subp_attr[i];
 
     if (u->type) {
       t = g_strdup_printf ("subtitle-%d-language", i);
@@ -525,8 +542,8 @@ gst_dvd_read_src_goto_title (GstDvdReadSrc * src, gint title, gint angle)
       lang_code[0] = '\0';
     }
 
-    GST_INFO_OBJECT (src, "[%02d] Subtitle %02d: lang='%s', format=%d",
-        src->title + 1, i, lang_code);
+    GST_INFO_OBJECT (src, "[%02d] Subtitle %02d: lang='%s', type=%d",
+        src->title + 1, i, lang_code, u->type);
   }
 
   src->title_lang_event_pending =
@@ -748,11 +765,14 @@ again:
     /* we loop until we're out of this cell */
     src->cur_pack = src->cur_pgc->cell_playback[src->cur_cell].first_sector;
     src->new_cell = FALSE;
+    GST_DEBUG_OBJECT (src, "Starting new cell %d @ pack %d", src->cur_cell,
+        src->cur_pack);
   }
 
   if (src->cur_pack >= src->cur_pgc->cell_playback[src->cur_cell].last_sector) {
     src->new_cell = TRUE;
-    GST_LOG_OBJECT (src, "Beyond last sector, go to next cell");
+    GST_LOG_OBJECT (src, "Beyond last sector for cell %d, going to next cell",
+        src->cur_cell);
     return GST_DVD_READ_AGAIN;
   }
 
@@ -764,6 +784,7 @@ nav_retry:
     goto read_error;
 
   if (!gst_dvd_read_src_is_nav_pack (oneblock)) {
+    GST_LOG_OBJECT (src, "Skipping nav packet @ pack %d", src->cur_pack);
     src->cur_pack++;
     goto nav_retry;
   }
@@ -775,7 +796,7 @@ nav_retry:
   /* determine where we go next. These values are the ones we
    * mostly care about */
   next_ilvu_start = src->cur_pack + dsi_pack.sml_agli.data[angle].address;
-  cur_output_size = dsi_pack.dsi_gi.vobu_ea;
+  cur_output_size = dsi_pack.dsi_gi.vobu_ea + 1;
 
   /* If we're not at the end of this cell, we can determine the next
    * VOBU to display using the VOBU_SRI information section of the
@@ -788,15 +809,17 @@ nav_retry:
   if (dsi_pack.vobu_sri.next_vobu != SRI_END_OF_CELL) {
     next_vobu = src->cur_pack + (dsi_pack.vobu_sri.next_vobu & 0x7fffffff);
   } else {
-    next_vobu = src->cur_pack + cur_output_size + 1;
+    next_vobu = src->cur_pgc->cell_playback[src->cur_cell].last_sector + 1;
   }
 
   g_assert (cur_output_size < 1024);
-  ++src->cur_pack;
 
   /* create the buffer (TODO: use buffer pool?) */
   buf = gst_buffer_new_and_alloc (cur_output_size * DVD_VIDEO_LB_LEN);
 
+  GST_LOG_OBJECT (src, "Going to read %u sectors @ pack %d", cur_output_size,
+      src->cur_pack);
+
   /* read in and output cursize packs */
   len = DVDReadBlocks (src->dvd_title, src->cur_pack, cur_output_size,
       GST_BUFFER_DATA (buf));
@@ -1051,7 +1074,7 @@ gst_dvd_read_src_handle_seek_event (GstDvdReadSrc * src, GstEvent * event)
     if (new_off < 0 || new_off >= src->num_angles) {
       GST_OBJECT_UNLOCK (src);
       GST_DEBUG_OBJECT (src, "invalid angle %d, only %d available",
-          src->num_angles);
+          src->num_angles, src->num_angles);
       return FALSE;
     }
     src->angle = (gint) new_off;
@@ -1339,6 +1362,102 @@ gst_dvd_read_src_do_position_query (GstDvdReadSrc * src, GstQuery * query)
   return TRUE;
 }
 
+static gboolean
+gst_dvd_read_src_do_convert_query (GstDvdReadSrc * src, GstQuery * query)
+{
+  GstFormat src_format, dest_format;
+  gboolean ret = FALSE;
+  gint64 src_val, dest_val = -1;
+
+  gst_query_parse_convert (query, &src_format, &src_val, &dest_format, NULL);
+
+  if (src_format == dest_format) {
+    dest_val = src_val;
+    ret = TRUE;
+    goto done;
+  }
+
+  /* Formats to consider: TIME, DEFAULT, BYTES, title, chapter, sector.
+   * Note: title and chapter are counted as starting from 0 here, just like
+   * in the context of seek events. Another note: DEFAULT format is undefined */
+
+  if (src_format == GST_FORMAT_BYTES) {
+    src_format = sector_format;
+    src_val /= DVD_VIDEO_LB_LEN;
+  }
+
+  if (src_format == sector_format) {
+    /* SECTOR => xyz */
+    if (dest_format == GST_FORMAT_TIME && src_val < G_MAXUINT) {
+      dest_val = gst_dvd_read_src_get_time_for_sector (src, (guint) src_val);
+      ret = (dest_val >= 0);
+    } else if (dest_format == GST_FORMAT_BYTES) {
+      dest_val = src_val * DVD_VIDEO_LB_LEN;
+      ret = TRUE;
+    } else {
+      ret = FALSE;
+    }
+  } else if (src_format == title_format) {
+    /* TITLE => xyz */
+    if (dest_format == GST_FORMAT_TIME) {
+      /* not really true, but we use this to trick the base source into
+       * handling seeks in title-format for us (the source won't know that
+       * we changed the title in this case) (changing titles should really
+       * be done with an interface rather than a seek, but for now we're
+       * stuck with this mechanism. Fix in 0.11) */
+      dest_val = (GstClockTime) 0;
+      ret = TRUE;
+    } else {
+      ret = FALSE;
+    }
+  } else if (src_format == chapter_format) {
+    /* CHAPTER => xyz */
+    if (dest_format == GST_FORMAT_TIME) {
+      if (src->num_chapters >= 0 && src_val < src->num_chapters) {
+        dest_val = src->chapter_starts[src_val];
+        ret = TRUE;
+      }
+    } else if (dest_format == sector_format) {
+    } else {
+      ret = FALSE;
+    }
+  } else if (src_format == GST_FORMAT_TIME) {
+    /* TIME => xyz */
+    if (dest_format == sector_format || dest_format == GST_FORMAT_BYTES) {
+      dest_val = gst_dvd_read_src_get_sector_from_time (src, src_val);
+      ret = (dest_val >= 0);
+      if (dest_format == GST_FORMAT_BYTES)
+        dest_val *= DVD_VIDEO_LB_LEN;
+    } else if (dest_format == chapter_format) {
+      if (src->chapter_starts != NULL) {
+        gint i;
+
+        for (i = src->num_chapters - 1; i >= 0; --i) {
+          if (src->chapter_starts && src->chapter_starts[i] >= src_val) {
+            dest_val = i;
+            ret = TRUE;
+            break;
+          }
+        }
+      } else {
+        ret = FALSE;
+      }
+    } else {
+      ret = FALSE;
+    }
+  } else {
+    ret = FALSE;
+  }
+
+done:
+
+  if (ret) {
+    gst_query_set_convert (query, src_format, src_val, dest_format, dest_val);
+  }
+
+  return ret;
+}
+
 static gboolean
 gst_dvd_read_src_src_query (GstBaseSrc * basesrc, GstQuery * query)
 {
@@ -1369,6 +1488,11 @@ gst_dvd_read_src_src_query (GstBaseSrc * basesrc, GstQuery * query)
       res = gst_dvd_read_src_do_position_query (src, query);
       GST_OBJECT_UNLOCK (src);
       break;
+    case GST_QUERY_CONVERT:
+      GST_OBJECT_LOCK (src);
+      res = gst_dvd_read_src_do_convert_query (src, query);
+      GST_OBJECT_UNLOCK (src);
+      break;
     default:
       res = GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
       break;
