@@ -42,9 +42,15 @@ G_BEGIN_DECLS
 typedef struct _GstLame GstLame;
 typedef struct _GstLameClass GstLameClass;
 
+/**
+ * GstLame:
+ *
+ * Opaque data structure.
+ */
 struct _GstLame {
   GstElement element;
-  /* pads */
+
+  /*< private >*/
   GstPad *srcpad, *sinkpad;
 
   gint samplerate;
@@ -88,7 +94,7 @@ struct _GstLame {
   lame_global_flags *lgf;
 
   /* time tracker */
-  guint64 last_ts, last_offs, last_duration;
+  guint64 last_ts, last_offs, last_duration, eos_ts;
 };
 
 struct _GstLameClass {
