@@ -38,6 +38,18 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_STATIC_CAPS ("video/x-ms-asf")
     );
 
+static GstStaticPadTemplate audio_src_template =
+GST_STATIC_PAD_TEMPLATE ("audio_%02d",
+    GST_PAD_SRC,
+    GST_PAD_SOMETIMES,
+    GST_STATIC_CAPS_ANY);
+
+static GstStaticPadTemplate video_src_template =
+GST_STATIC_PAD_TEMPLATE ("video_%02d",
+    GST_PAD_SRC,
+    GST_PAD_SOMETIMES,
+    GST_STATIC_CAPS_ANY);
+
 
 /* abuse this GstFlowReturn enum for internal usage */
 #define ASF_FLOW_NEED_MORE_DATA  99
@@ -65,9 +77,6 @@ static GstFlowReturn gst_asf_demux_process_object (GstASFDemux * demux,
 
 GST_BOILERPLATE (GstASFDemux, gst_asf_demux, GstElement, GST_TYPE_ELEMENT);
 
-static GstPadTemplate *videosrctempl;
-static GstPadTemplate *audiosrctempl;
-
 static void
 gst_asf_demux_base_init (gpointer g_class)
 {
@@ -78,16 +87,11 @@ gst_asf_demux_base_init (gpointer g_class)
     "Demultiplexes ASF Streams",
     "Owen Fraser-Green <owen@discobabe.net>"
   };
-  GstCaps *audcaps = gst_riff_create_audio_template_caps (),
-      *vidcaps = gst_riff_create_video_template_caps ();
 
-  audiosrctempl = gst_pad_template_new ("audio_%02d",
-      GST_PAD_SRC, GST_PAD_SOMETIMES, audcaps);
-  videosrctempl = gst_pad_template_new ("video_%02d",
-      GST_PAD_SRC, GST_PAD_SOMETIMES, vidcaps);
-
-  gst_element_class_add_pad_template (element_class, audiosrctempl);
-  gst_element_class_add_pad_template (element_class, videosrctempl);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&audio_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&video_src_template));
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_asf_demux_sink_template));
 
@@ -148,7 +152,12 @@ gst_asf_demux_sink_event (GstPad * pad, GstEvent * event)
       gst_event_parse_new_segment (event, NULL, NULL, &newsegment_format,
           &newsegment_start, NULL, NULL);
 
-      g_assert (newsegment_format == GST_FORMAT_BYTES);
+      if (newsegment_format != GST_FORMAT_BYTES) {
+        GST_WARNING_OBJECT (demux, "newsegment format not BYTES, ignoring");
+        gst_event_unref (event);
+        break;
+      }
+
       g_assert (newsegment_start >= 0);
 
       GST_OBJECT_LOCK (demux);
@@ -865,6 +874,7 @@ gst_asf_demux_setup_pad (GstASFDemux * demux, GstPad * src_pad,
 
   ++demux->num_streams;
 
+  gst_pad_set_active (src_pad, TRUE);
   gst_element_add_pad (GST_ELEMENT (demux), src_pad);
 }
 
@@ -886,13 +896,13 @@ gst_asf_demux_add_audio_stream (GstASFDemux * demux,
   /* Create the audio pad */
   name = g_strdup_printf ("audio_%02d", demux->num_audio_streams);
 
-  src_pad = gst_pad_new_from_template (audiosrctempl, name);
+  src_pad = gst_pad_new_from_static_template (&audio_src_template, name);
   g_free (name);
 
   /* Swallow up any left over data and set up the 
    * standard properties from the header info */
   if (size_left) {
-    GST_WARNING ("Audio header contains %d bytes of "
+    GST_INFO_OBJECT (demux, "Audio header contains %d bytes of "
         "codec specific data", size_left);
 
     gst_asf_demux_get_buffer (&extradata, size_left, p_data, p_size);
@@ -951,7 +961,7 @@ gst_asf_demux_add_video_stream (GstASFDemux * demux,
 
   /* Create the video pad */
   name = g_strdup_printf ("video_%02d", demux->num_video_streams);
-  src_pad = gst_pad_new_from_template (videosrctempl, name);
+  src_pad = gst_pad_new_from_static_template (&video_src_template, name);
   g_free (name);
 
   /* Now try some gstreamer formatted MIME types (from gst_avi_demux_strf_vids) */
@@ -1152,7 +1162,7 @@ gst_asf_demux_get_gst_tag_from_tag_name (const gchar * name_utf16le,
     "WM/Genre", GST_TAG_GENRE}, {
     "WM/AlbumTitle", GST_TAG_ALBUM}, {
     "WM/AlbumArtist", GST_TAG_ARTIST}, {
-    "WM/TrackNumber", GST_TAG_TRACK_NUMBER}, {
+    "WM/Track", GST_TAG_TRACK_NUMBER}, {
     "WM/Year", GST_TAG_DATE}
     /* { "WM/Composer", GST_TAG_COMPOSER } */
   };
@@ -1171,11 +1181,13 @@ gst_asf_demux_get_gst_tag_from_tag_name (const gchar * name_utf16le,
 
   for (i = 0; i < G_N_ELEMENTS (tags); ++i) {
     if (strncmp (tags[i].asf_name, name_utf8, out) == 0) {
+      GST_LOG ("map tagname '%s' -> '%s'", name_utf8, tags[i].gst_name);
       g_free (name_utf8);
       return tags[i].gst_name;
     }
   }
 
+  GST_LOG ("unhandled tagname '%s'", name_utf8);
   g_free (name_utf8);
   return NULL;
 }
@@ -1221,10 +1233,10 @@ gst_asf_demux_process_ext_content_desc (GstASFDemux * demux, guint8 ** p_data,
    *   WM/ProviderRating = 8
    *   WM/ProviderStyle = Rock (similar to WM/Genre)
    *   WM/GenreID (similar to WM/Genre)
+   *   WM/TrackNumber (same as WM/Track but as a string)
    *
    * Other known (and unused) 'non-text' metadata available :
    *
-   *   WM/Track (same as WM/TrackNumber but starts at 0)
    *   WM/EncodingTime
    *   WM/MCDI
    *   IsVBR
@@ -1304,8 +1316,9 @@ gst_asf_demux_process_ext_content_desc (GstASFDemux * demux, guint8 ** p_data,
           break;
         }
         case ASF_DEMUX_DATA_TYPE_DWORD:{
-          g_value_init (&tag_value, G_TYPE_INT);
-          g_value_set_int (&tag_value, (gint) GST_READ_UINT32_LE (value));
+          /* this is the track number */
+          g_value_init (&tag_value, G_TYPE_UINT);
+          g_value_set_uint (&tag_value, (guint) GST_READ_UINT32_LE (value));
           break;
         }
         default:{
@@ -1334,7 +1347,7 @@ gst_asf_demux_process_ext_content_desc (GstASFDemux * demux, guint8 ** p_data,
 
   return GST_FLOW_OK;
 
-
+  /* Errors */
 not_enough_data:
   {
     /* avoid compiler warning when disabling logging at compile time */
@@ -2380,7 +2393,7 @@ gst_asf_demux_process_segment (GstASFDemux * demux,
 
     time_start = segment_info.frag_offset;
     segment_info.frag_offset = 0;
-    segment_info.frag_timestamp = time_start;
+    segment_info.frag_timestamp = time_start;   /* was: demux->timestamp */
   }
 
   GST_DEBUG ("multiple = %u, compressed = %u",
@@ -2607,7 +2620,7 @@ gst_asf_demux_parse_data (GstASFDemux * demux)
 
       data_left = (guint64) gst_adapter_available (demux->adapter);
 
-      GST_DEBUG ("STATE_HEADER, avail=%u:", data_left);
+      GST_DEBUG ("STATE_HEADER, avail=%u", (guint) data_left);
 
       data = (guint8 *) gst_adapter_peek (demux->adapter, data_left);
 
