@@ -27,6 +27,7 @@
 #  include "config.h"
 #endif
 #include "rmdemux.h"
+#include "rdtdepay.h"
 #include "rmutils.h"
 #include <string.h>
 #include <ctype.h>
@@ -134,22 +135,22 @@ static gboolean gst_rmdemux_src_query (GstPad * pad, GstQuery * query);
 static gboolean gst_rmdemux_perform_seek (GstRMDemux * rmdemux,
     GstEvent * event);
 
-static void gst_rmdemux_parse__rmf (GstRMDemux * rmdemux, const void *data,
+static void gst_rmdemux_parse__rmf (GstRMDemux * rmdemux, const guint8 * data,
     int length);
-static void gst_rmdemux_parse_prop (GstRMDemux * rmdemux, const void *data,
+static void gst_rmdemux_parse_prop (GstRMDemux * rmdemux, const guint8 * data,
     int length);
 static void gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux,
-    const void *data, int length);
-static guint gst_rmdemux_parse_indx (GstRMDemux * rmdemux, const void *data,
+    const guint8 * data, int length);
+static guint gst_rmdemux_parse_indx (GstRMDemux * rmdemux, const guint8 * data,
     int length);
-static void gst_rmdemux_parse_data (GstRMDemux * rmdemux, const void *data,
+static void gst_rmdemux_parse_data (GstRMDemux * rmdemux, const guint8 * data,
     int length);
-static void gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const void *data,
+static void gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const guint8 * data,
     int length);
 static GstFlowReturn gst_rmdemux_parse_packet (GstRMDemux * rmdemux,
-    const void *data, guint16 version, guint16 length);
-static void gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const void *data,
-    int length);
+    const guint8 * data, guint16 version, guint16 length);
+static void gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux,
+    const guint8 * data, int length);
 static void gst_rmdemux_stream_clear_cached_subpackets (GstRMDemux * rmdemux,
     GstRMDemuxStream * stream);
 static GstRMDemuxStream *gst_rmdemux_get_stream_by_id (GstRMDemux * rmdemux,
@@ -814,9 +815,9 @@ gst_rmdemux_loop (GstPad * pad)
       GST_OBJECT_UNLOCK (rmdemux);
       return;
     } else {
-      GST_DEBUG_OBJECT (rmdemux,
-          "Unable to pull %d bytes at offset %p (pull_range returned %d, state is %d)",
-          (int) size, rmdemux->offset, ret, GST_STATE (rmdemux));
+      GST_DEBUG_OBJECT (rmdemux, "Unable to pull %d bytes at offset 0x%08x "
+          "(pull_range returned flow %s, state is %d)", (gint) size,
+          rmdemux->offset, gst_flow_get_name (ret), GST_STATE (rmdemux));
       goto need_pause;
     }
   }
@@ -1410,7 +1411,8 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 
     GST_DEBUG_OBJECT (rmdemux, "adding pad %s with caps %" GST_PTR_FORMAT
         ", stream_id=%d", GST_PAD_NAME (stream->pad), stream_caps, stream->id);
-    gst_element_add_pad (GST_ELEMENT (rmdemux), stream->pad);
+    gst_pad_set_active (stream->pad, TRUE);
+    gst_element_add_pad (GST_ELEMENT_CAST (rmdemux), stream->pad);
 
     gst_pad_push_event (stream->pad,
         gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, (gint64) 0,
@@ -1431,36 +1433,6 @@ beach:
     gst_caps_unref (stream_caps);
 }
 
-G_GNUC_UNUSED static void
-re_hexdump_bytes (guint8 * ptr, int len, int offset)
-{
-  guint8 *end = ptr + len;
-  int i;
-
-  while (1) {
-    if (ptr >= end)
-      return;
-    g_print ("%08x: ", offset);
-    for (i = 0; i < 16; i++) {
-      if (ptr + i >= end) {
-        g_print ("   ");
-      } else {
-        g_print ("%02x ", ptr[i]);
-      }
-    }
-    for (i = 0; i < 16; i++) {
-      if (ptr + i >= end) {
-        g_print (" ");
-      } else {
-        g_print ("%c", g_ascii_isprint (ptr[i]) ? ptr[i] : '.');
-      }
-    }
-    g_print ("\n");
-    ptr += 16;
-    offset += 16;
-  }
-}
-
 static int
 re_skip_pascal_string (const guint8 * ptr)
 {
@@ -1472,14 +1444,14 @@ re_skip_pascal_string (const guint8 * ptr)
 }
 
 static void
-gst_rmdemux_parse__rmf (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse__rmf (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   GST_LOG_OBJECT (rmdemux, "file_version: %d", RMDEMUX_GUINT32_GET (data));
   GST_LOG_OBJECT (rmdemux, "num_headers: %d", RMDEMUX_GUINT32_GET (data + 4));
 }
 
 static void
-gst_rmdemux_parse_prop (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_prop (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   GST_LOG_OBJECT (rmdemux, "max bitrate: %d", RMDEMUX_GUINT32_GET (data));
   GST_LOG_OBJECT (rmdemux, "avg bitrate: %d", RMDEMUX_GUINT32_GET (data + 4));
@@ -1505,7 +1477,7 @@ gst_rmdemux_parse_prop (GstRMDemux * rmdemux, const void *data, int length)
 }
 
 static void
-gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   GstRMDemuxStream *stream;
   char *stream1_type_string;
@@ -1546,8 +1518,17 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const void *data, int length)
 
   if (strcmp (stream2_type_string, "video/x-pn-realvideo") == 0) {
     stream_type = GST_RMDEMUX_STREAM_VIDEO;
+  } else if (strcmp (stream2_type_string,
+          "video/x-pn-multirate-realvideo") == 0) {
+    stream_type = GST_RMDEMUX_STREAM_VIDEO;
   } else if (strcmp (stream2_type_string, "audio/x-pn-realaudio") == 0) {
     stream_type = GST_RMDEMUX_STREAM_AUDIO;
+  } else if (strcmp (stream2_type_string,
+          "audio/x-pn-multirate-realaudio") == 0) {
+    stream_type = GST_RMDEMUX_STREAM_AUDIO;
+  } else if (strcmp (stream2_type_string,
+          "audio/x-pn-multirate-realaudio-live") == 0) {
+    stream_type = GST_RMDEMUX_STREAM_AUDIO;
   } else if (strcmp (stream2_type_string, "audio/x-ralf-mpeg4-generic") == 0) {
     /* Another audio type found in the real testsuite */
     stream_type = GST_RMDEMUX_STREAM_AUDIO;
@@ -1624,8 +1605,15 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const void *data, int length)
           stream->sample_width = RMDEMUX_GUINT16_GET (data + offset + 52);
           stream->n_channels = RMDEMUX_GUINT16_GET (data + offset + 54);
           stream->fourcc = RMDEMUX_FOURCC_GET (data + offset + 62);
-          stream->extra_data_size = 16;
-          stream->extra_data = (guint8 *) data + offset + 71;
+          stream->extra_data_size = RMDEMUX_GUINT32_GET (data + offset + 69);
+          GST_DEBUG_OBJECT (rmdemux, "%u bytes of extra codec data",
+              stream->extra_data_size);
+          if (length - (offset + 73) >= stream->extra_data_size) {
+            stream->extra_data = (guint8 *) data + offset + 73;
+          } else {
+            GST_WARNING_OBJECT (rmdemux, "codec data runs beyond MDPR chunk");
+            stream->extra_data_size = 0;
+          }
           break;
         case 5:
           stream->flavor = RMDEMUX_GUINT16_GET (data + offset + 22);
@@ -1638,7 +1626,14 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const void *data, int length)
           stream->n_channels = RMDEMUX_GUINT16_GET (data + offset + 60);
           stream->fourcc = RMDEMUX_FOURCC_GET (data + offset + 66);
           stream->extra_data_size = RMDEMUX_GUINT32_GET (data + offset + 74);
-          stream->extra_data = (guint8 *) data + offset + 78;
+          GST_DEBUG_OBJECT (rmdemux, "%u bytes of extra codec data",
+              stream->extra_data_size);
+          if (length - (offset + 78) >= stream->extra_data_size) {
+            stream->extra_data = (guint8 *) data + offset + 78;
+          } else {
+            GST_WARNING_OBJECT (rmdemux, "codec data runs beyond MDPR chunk");
+            stream->extra_data_size = 0;
+          }
           break;
         default:{
           GST_WARNING_OBJECT (rmdemux, "Unhandled audio stream version %d",
@@ -1698,7 +1693,7 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const void *data, int length)
 }
 
 static guint
-gst_rmdemux_parse_indx (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_indx (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   int n;
   int id;
@@ -1718,7 +1713,8 @@ gst_rmdemux_parse_indx (GstRMDemux * rmdemux, const void *data, int length)
 }
 
 static void
-gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const guint8 * data,
+    int length)
 {
   int i;
   int n;
@@ -1745,7 +1741,7 @@ gst_rmdemux_parse_indx_data (GstRMDemux * rmdemux, const void *data, int length)
 }
 
 static void
-gst_rmdemux_parse_data (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_data (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   rmdemux->n_chunks = RMDEMUX_GUINT32_GET (data);
   rmdemux->data_offset = RMDEMUX_GUINT32_GET (data + 4);
@@ -1755,7 +1751,7 @@ gst_rmdemux_parse_data (GstRMDemux * rmdemux, const void *data, int length)
 }
 
 static void
-gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const void *data, int length)
+gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const guint8 * data, int length)
 {
   GstTagList *tags;
 
@@ -1920,7 +1916,7 @@ gst_rmdemux_handle_scrambled_packet (GstRMDemux * rmdemux,
 }
 
 static GstFlowReturn
-gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const void *data,
+gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const guint8 * data,
     guint16 version, guint16 length)
 {
   guint16 id;
@@ -1964,7 +1960,7 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const void *data,
 
   if ((rmdemux->offset + packet_size) <= stream->seek_offset) {
     GST_DEBUG_OBJECT (rmdemux,
-        "Stream %d is skipping: seek_offset=%d, offset=%d, packet_size",
+        "Stream %d is skipping: seek_offset=%d, offset=%d, packet_size=%u",
         stream->id, stream->seek_offset, rmdemux->offset, packet_size);
     cret = GST_FLOW_OK;
     goto beach;
@@ -1975,7 +1971,7 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, const void *data,
 
   cret = gst_rmdemux_combine_flows (rmdemux, stream, ret);
   if (ret != GST_FLOW_OK)
-    goto beach;
+    goto alloc_failed;
 
   memcpy (GST_BUFFER_DATA (buffer), (guint8 *) data, packet_size);
   GST_BUFFER_TIMESTAMP (buffer) = timestamp;
@@ -2001,17 +1997,35 @@ unknown_stream:
         "data packet", id);
     return GST_FLOW_OK;
   }
+alloc_failed:
+  {
+    GST_DEBUG_OBJECT (rmdemux, "pad alloc returned %d", ret);
+    return cret;
+  }
 }
 
-static gboolean
-plugin_init (GstPlugin * plugin)
+gboolean
+gst_rmdemux_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rmdemux",
       GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
 }
 
+
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  if (!gst_rmdemux_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rdt_depay_plugin_init (plugin))
+    return FALSE;
+
+  return TRUE;
+}
+
 GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
     GST_VERSION_MINOR,
-    "rmdemux",
-    "Realmedia stream demuxer",
+    "realmedia",
+    "RealMedia demuxer and depayloader",
     plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
